﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities.Resources;

namespace Palmmedia.MvcSampleApplication.Core.Domain.Entities
{
    /// <summary>
    /// Extends validation based on <see cref="IDataErrorInfo"/>.
    /// </summary>
    public partial class GuestbookEntry : IDataErrorInfo
    {
        /// <summary>
        /// String value representing the 'Comment' property
        /// </summary>
        private const string CommentProperty = "Comment";

        /// <summary>
        /// Gets an array containing all properties to validate.
        /// </summary>
        private static readonly string[] ValidatedProperties = 
        { 
            CommentProperty
        };

        /// <summary>
        /// Initializes a new instance of the <see cref="GuestbookEntry"/> class.
        /// </summary>
        public GuestbookEntry()
        {
            this.Id = Guid.NewGuid();
            this.Created = DateTime.Now;
        }

        /// <summary>
        /// Gets an error message indicating what is wrong with this object.
        /// </summary>
        /// <returns>An error message indicating what is wrong with this object. The default is an empty string ("").</returns>
        public string Error
        {
            get { return string.Empty; }
        }

        /// <summary>
        /// Gets a value indicating whether this object has validation errors.
        /// </summary>
        public bool IsValid
        {
            get
            {
                foreach (string property in ValidatedProperties)
                {
                    if (this[property] != null)
                    {
                        return false;
                    }
                }

                return true;
            }
        }

        /// <summary>
        /// Gets the error of the specified column name.
        /// </summary>
        /// <param name="columnName">The name of the property</param>
        public string this[string columnName]
        {
            get 
            {
                if (Array.IndexOf(ValidatedProperties, columnName) < 0)
                {
                    return null;
                }

                string error = null;

                switch (columnName)
                {
                    case CommentProperty:
                        error = this.ValidateComment();
                        break;

                    default:
                        Debug.Fail("Unexpected property being validated on GuestbookEntry: " + columnName);
                        break;
                }

                return error;
            }
        }

        /// <summary>
        /// Validated the 'Comment' property
        /// </summary>
        /// <returns>The error or null if none exists.</returns>
        private string ValidateComment()
        {
            if (string.IsNullOrEmpty(this.Comment))
            {
                return GuestbookEntryResources.Comment;
            }
            else
            {
                return null;
            }
        }
    }
}
