﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;

namespace Palmmedia.MvcSampleApplication.Service.WcfFacade
{
    /// <summary>
    /// Interface for the the service facade.
    /// </summary>
    [ServiceContract(Namespace = "http://www.palmmedia.de/mvcsampleapplication")]
    public interface IServiceFacade
    {
        /// <summary>
        /// Adds the given <see cref="User"/>.
        /// </summary>
        /// <param name="user">The <see cref="User"/> to add.</param>
        [OperationContract]
        void AddUser(User user);

        /// <summary>
        /// Gets the <see cref="User"/> with the given Id.
        /// </summary>
        /// <param name="id">The Id of the <see cref="User"/>.</param>
        /// <returns>The <see cref="User"/> with the given Id.</returns>
        [OperationContract]
        User GetUser(Guid id);

        /// <summary>
        /// Gets the <see cref="User"/> with the given name.
        /// </summary>
        /// <param name="name">The name of the <see cref="User"/>.</param>
        /// <returns>The <see cref="User"/> with the given name.</returns>
        [OperationContract(Name = "GetUserByName")]
        User GetUser(string name);

        /// <summary>
        /// Determinates whether an <see cref="User"/> with the given name and password exists.
        /// </summary>
        /// <param name="name">The name of the <see cref="User"/>.</param>
        /// <param name="password">The password of th <see cref="User"/>.</param>
        /// <returns>True if the <see cref="User"/> exists, otherwise false.</returns>
        [OperationContract]
        bool UserExists(string name, string password);

        /// <summary>
        /// Updates the given <see cref="User"/>.
        /// </summary>
        /// <param name="user">The <see cref="User"/> to update.</param>
        [OperationContract]
        void UpdateUser(User user);

        /// <summary>
        /// Gets all <see cref="User">Users</see>.
        /// </summary>
        /// <returns>All <see cref="User">Users</see>.</returns>
        [OperationContract]
        IEnumerable<User> GetUsers();

        /// <summary>
        /// Adds the given <see cref="GuestbookEntry"/>.
        /// </summary>
        /// <param name="guestbookEntry">The <see cref="GuestbookEntry"/> to add.</param>
        [OperationContract]
        void AddGuestbookEntry(GuestbookEntry guestbookEntry);

        /// <summary>
        /// Gets all <see cref="GuestbookEntry">GuestbookEntries</see>.
        /// </summary>
        /// <returns>All <see cref="GuestbookEntry">GuestbookEntries</see>.</returns>
        [OperationContract]
        IEnumerable<GuestbookEntry> GetGuestbookEntries();

        /// <summary>
        /// Gets the <see cref="GuestbookEntry"/> with the given Id.
        /// </summary>
        /// <param name="id">The Id of the <see cref="GuestbookEntry"/>.</param>
        /// <returns>The <see cref="GuestbookEntry"/>.</returns>
        [OperationContract]
        GuestbookEntry GetGuestbookEntry(Guid id);

        /// <summary>
        /// Gets all <see cref="GuestbookEntry">GuestbookEntries</see> of the the <see cref="User"/> with the given Id.
        /// </summary>
        /// <param name="userId">The Id of the <see cref="User"/>.</param>
        /// <returns>All <see cref="GuestbookEntry">GuestbookEntries</see> of the the <see cref="User"/>.</returns>
        [OperationContract]
        IEnumerable<GuestbookEntry> GetGuestbookEntriesByUser(Guid userId);

        /// <summary>
        /// Updates the given <see cref="GuestbookEntry"/>.
        /// </summary>
        /// <param name="guestbookEntry">The <see cref="GuestbookEntry"/> to update.</param>
        [OperationContract]
        void UpdateGuestbookEntry(GuestbookEntry guestbookEntry);

        /// <summary>
        /// Deletes the <see cref="GuestbookEntry"/> with the given Id.
        /// </summary>
        /// <param name="id">The Id of the <see cref="GuestbookEntry"/>.</param>
        [OperationContract]
        void DeleteGuestbookEntry(Guid id);
    }
}
