﻿using System.Web.Mvc;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;
using Palmmedia.MvcSampleApplication.Service.Proxy.ServiceReference;

namespace Palmmedia.MvcSampleApplication.UI.AspMvc.Authorization
{
    /// <summary>
    /// Provides easy access to currently logged in <see cref="Palmmedia.MvcSampleApplication.Core.Domain.Entities.User">user</see>.
    /// </summary>
    public static class AuthenticationExtensions
    {
        /// <summary>
        /// Gets the currently logged in <see cref="Palmmedia.MvcSampleApplication.Core.Domain.Entities.User">user</see> or null if no user is logged in.
        /// </summary>
        /// <param name="controller">The <see cref="Controller"/>.</param>
        /// <returns>The <see cref="Palmmedia.MvcSampleApplication.Core.Domain.Entities.User">user</see> or null if no user is logged in.</returns>
        public static User GetUser(this Controller controller)
        {
            if (string.IsNullOrEmpty(controller.User.Identity.Name))
            {
                return null;
            }

            var service = Palmmedia.Common.Ioc.IocHelper.Resolve<IServiceFacade>();
            
            return service.GetUserByName(controller.User.Identity.Name);
        }
    }
}