﻿using System;
using System.Web.Mvc;
using Palmmedia.Common.Ioc;

namespace Palmmedia.MvcSampleApplication.UI.AspMvc.Controllers
{
    /// <summary>
    /// Custom <see cref="IControllerFactory"/> which uses dependcy injection to instantiate controllers.
    /// </summary>
    public class IocControllerFactory : DefaultControllerFactory   
    {
        /// <summary>
        /// Retrieves the controller instance for the specified request context and controller type.
        /// </summary>
        /// <param name="requestContext">The context of the HTTP request, which includes the HTTP context and route data.</param>
        /// <param name="controllerType">The type of the controller.</param>
        /// <returns>The controller instance.</returns>
        /// <exception cref="T:System.Web.HttpException">
        /// 	<paramref name="controllerType"/> is null.</exception>
        /// <exception cref="T:System.ArgumentException">
        /// 	<paramref name="controllerType"/> cannot be assigned.</exception>
        /// <exception cref="T:System.InvalidOperationException">An instance of <paramref name="controllerType"/> cannot be created.</exception>
        protected override IController GetControllerInstance(System.Web.Routing.RequestContext requestContext, Type controllerType)
        {
            if (controllerType == null)
            {
                return base.GetControllerInstance(requestContext, controllerType);
            }

            return (Controller)IocHelper.Resolve(controllerType);  
        }
    }  
}