﻿using System;
using System.Web.Mvc;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;

namespace Palmmedia.MvcSampleApplication.UI.AspMvc.Controllers
{
    /// <summary>
    /// Controller for user management.
    /// </summary>
    [Authorize(Roles = "Admin")]
    [Palmmedia.Common.Net.Mvc.HandleError]
    public partial class UserController : ControllerBase
    {
        /// <summary>
        /// Shows the user overview.
        /// </summary>
        /// <returns>The user overview.</returns>
        public virtual ActionResult Index()
        {
            var users = this.ServiceFacade.GetUsers();

            return View(users);
        }

        /// <summary>
        /// Shows a view, where a user can be edited.
        /// </summary>
        /// <param name="id">The Id of the user.</param>
        /// <returns>View showing a single user.</returns>
        public virtual ActionResult Edit(Guid? id)
        {
            if (id == null)
            {
                return View(MVC.User.Views.NotFound);
            }

            Palmmedia.MvcSampleApplication.Core.Domain.Entities.User user = this.ServiceFacade.GetUser(id.Value);

            if (user == null)
            {
                return View(MVC.User.Views.NotFound);
            }

            return View(user);
        }

        /// <summary>
        /// Saves the edited user.
        /// </summary>
        /// <param name="id">The Id of the user.</param>
        /// <param name="name">The name</param>
        /// <param name="password">The password.</param>
        /// <returns>The user overview.</returns>
        [ValidateInput(false)] 
        [AcceptVerbs(HttpVerbs.Post)]
        public virtual ActionResult Edit(Guid? id, string name, string password)
        {
            if (id == null)
            {
                return View(MVC.User.Views.NotFound);
            }

            User user = this.ServiceFacade.GetUser(id.Value);

            if (user == null)
            {
                return View(MVC.User.Views.NotFound);
            }

            try
            {
                this.UpdateModel<User>(user, new string[] { "Name", "Password" });
            }
            catch (InvalidOperationException)
            {
                return View();
            }

            this.UpdateModel(user, new string[] { "Name", "Password" });
            this.ServiceFacade.UpdateUser(user);

            return this.RedirectToAction(MVC.User.Index());
        }
    }
}
