﻿using System;
using System.Globalization;
using System.Threading;
using System.Windows;
using log4net;
using Palmmedia.MvcSampleApplication.UI.Wpf.ViewModels;
using Palmmedia.MvcSampleApplication.UI.Wpf.Windows;
using RootLibrary.WPF.Localization;

namespace Palmmedia.MvcSampleApplication.UI.Wpf
{
    /// <summary>
    /// Interaction logic for App.xaml.
    /// </summary>
    public partial class App : Application
    {
        /// <summary>
        /// Logger instance.
        /// </summary>
        private static readonly ILog logger = LogManager.GetLogger(typeof(App));

        /// <summary>
        /// Raises the <see cref="E:System.Windows.Application.Startup"/> event.
        /// </summary>
        /// <param name="e">A <see cref="T:System.Windows.StartupEventArgs"/> that contains the event data.</param>
        protected override void OnStartup(StartupEventArgs e)
        {
            Thread.CurrentThread.Name = "MainThread";
            LocalizeDictionary.Instance.Culture = Thread.CurrentThread.CurrentCulture;

            base.OnStartup(e);

            // Init logger
            log4net.Config.XmlConfigurator.Configure();

            this.DispatcherUnhandledException += new System.Windows.Threading.DispatcherUnhandledExceptionEventHandler(this.App_DispatcherUnhandledException);

            logger.Debug("Initializing main window.");
            var mainWindow = new MainWindow();
            mainWindow.DataContext = new MainWindowViewModel();

            // Apply application settings
            this.MainWindow = mainWindow;

            mainWindow.Show();
        }

        /// <summary>
        /// Switches the culture.
        /// </summary>
        /// <param name="culture">The culture.</param>
        public static void SwitchCulture(string culture)
        {
            var cultureInfo = CultureInfo.InvariantCulture;

            if (culture.Equals("de", StringComparison.OrdinalIgnoreCase))
            {
                cultureInfo = CultureInfo.CreateSpecificCulture("de");
            }

            LocalizeDictionary.Instance.Culture = cultureInfo;
            Thread.CurrentThread.CurrentCulture = cultureInfo;
            Thread.CurrentThread.CurrentUICulture = cultureInfo;
        }

        /// <summary>
        /// Handles the DispatcherUnhandledException event of the Application.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Threading.DispatcherUnhandledExceptionEventArgs"/> instance containing the event data.</param>
        private void App_DispatcherUnhandledException(object sender, System.Windows.Threading.DispatcherUnhandledExceptionEventArgs e)
        {
            logger.Error("Unhandled Exception occured.", e.Exception);
            MessageBox.Show(e.Exception.Message);
            e.Handled = true;
        }
    }
}
