using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace Palmmedia.MvcSampleApplication.UI.Wpf.Common
{
	public class MouseClickBehavior
	{
		public static DependencyProperty MouseClickProperty = DependencyProperty.RegisterAttached(
											"MouseClick",
											typeof(ICommand),
                                            typeof(MouseClickBehavior),
                                            new UIPropertyMetadata(MouseClickBehavior.DoubleClick));

		public static void SetMouseClick(DependencyObject target, ICommand value)
		{
			target.SetValue(MouseClickBehavior.MouseClickProperty, value);
		}

		private static void DoubleClick(DependencyObject target, DependencyPropertyChangedEventArgs e)
		{
			var element = target as Image;

			if (element == null)
				throw new InvalidOperationException("This behavior can be attached to a Image item only.");

			if ((e.NewValue != null) && (e.OldValue == null))
				element.MouseLeftButtonDown += new MouseButtonEventHandler(Element_MouseClick);
			else if ((e.NewValue == null) && (e.OldValue != null))
                    element.MouseLeftButtonDown -= new MouseButtonEventHandler(Element_MouseClick);
		}

		static void Element_MouseClick(object sender, MouseButtonEventArgs e)
		{
			UIElement element = (UIElement)sender;
			ICommand command = (ICommand)element.GetValue(MouseClickBehavior.MouseClickProperty);
			command.Execute(e);
		}
	}
}
