﻿using System;
using System.Globalization;
using System.Windows.Data;

namespace Palmmedia.MvcSampleApplication.UI.Wpf.Resources.Converters
{
    /// <summary>
    /// Converts a <see cref="DateTime"/> to a <see cref="string"/>.
    /// </summary>
    public class DateTimeToStringConverter : IValueConverter
    {
        /// <summary>
        /// Converts a <see cref="DateTime"/> to a <see cref="string"/>, if DateTime is empty "-" is returned.
        /// </summary>
        /// <param name="value">The value produced by the binding source.</param>
        /// <param name="targetType">The type of the binding target property.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        object IValueConverter.Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            DateTime dateTime = (DateTime)value;
            return dateTime.Equals(new DateTime()) ? "-" : dateTime.ToString();
        }

        /// <summary>
        /// Converts a <see cref="string"/> to a <see cref="DateTime"/>
        /// </summary>
        /// <param name="value">The value that is produced by the binding target.</param>
        /// <param name="targetType">The type to convert to.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        object IValueConverter.ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
