﻿using System.Collections.Generic;
using System.Windows.Input;
using Palmmedia.MvcSampleApplication.Core.Domain.Entities;
using Palmmedia.MvcSampleApplication.UI.Wpf.Common;

namespace Palmmedia.MvcSampleApplication.UI.Wpf.ViewModels
{
    /// <summary>
    /// ViewModel for <see cref="Palmmedia.MvcSampleApplication.UI.Wpf.Views.GuestbookView"/>.
    /// </summary>
    internal class GuestbookViewModel : ViewModelBase
    {
        /// <summary>
        /// The new comment.
        /// </summary>
        private GuestbookEntry comment;

        /// <summary>
        /// Executed when a <see cref="GuestbookEntry"/> should be added. 
        /// </summary>
        private ICommand addGuestbookEntryCommand;

        /// <summary>
        /// Initializes a new instance of the <see cref="GuestbookViewModel"/> class.
        /// </summary>
        public GuestbookViewModel()
        {
            this.Comment = new GuestbookEntry();
        }

        /// <summary>
        /// Gets the <see cref="GuestbookEntry">GuestbookEntries</see> which are displayed.
        /// </summary>
        public IEnumerable<GuestbookEntry> GuestbookEntries
        {
            get
            {
                return this.ServiceFacade.GetGuestbookEntries();
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="GuestbookEntry"/>which should be created.
        /// </summary>
        public GuestbookEntry Comment
        {
            get
            {
                return this.comment;
            }

            set
            {
                value.User = CurrentUser;
                this.comment = value;
                OnPropertyChanged("Comment");
            }
        }

        /// <summary>
        /// Gets the command which is executed when a <see cref="GuestbookEntry"/> should be added. 
        /// </summary>
        public ICommand AddGuestbookEntryCommand
        {
            get
            {
                if (this.addGuestbookEntryCommand == null)
                {
                    this.addGuestbookEntryCommand = new RelayCommand(param => this.AddGuestbookEntry(), param => this.CanAddGuestbookEntry);
                }

                return this.addGuestbookEntryCommand;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the new <see cref="GuestbookEntry"/> can be added. 
        /// </summary>
        private bool CanAddGuestbookEntry
        {
            get { return this.Comment.IsValid; }
        }

        /// <summary>
        /// Adds a new guestbook entry.
        /// </summary>
        private void AddGuestbookEntry()
        {
            this.ServiceFacade.AddGuestbookEntry(this.Comment);
            this.Comment = new GuestbookEntry();

            OnPropertyChanged("GuestbookEntries");
        }
    }
}
