﻿using System.Windows.Input;
using Palmmedia.MvcSampleApplication.UI.Wpf.Common;

namespace Palmmedia.MvcSampleApplication.UI.Wpf.ViewModels
{
	/// <summary>
	/// ViewModel for the MainWindow
	/// </summary>
    internal class MainWindowViewModel : ViewModelBase
    {
		/// <summary>
		/// The <see cref="ViewModelBase"/> which should be displayed.
		/// </summary>
		private ViewModelBase mainViewModel;

        /// <summary>
        /// Executed when a logout should be performed.
        /// </summary>
        private ICommand logoutCommand;

        /// <summary>
        /// Executed when language should be switched to German.
        /// </summary>
        private ICommand germanCommand;

        /// <summary>
        /// Executed when language should be switched to English.
        /// </summary>
        private ICommand englishCommand;

		/// <summary>
		/// Initializes a new instance of the <see cref="MainWindowViewModel" /> class.
		/// </summary>
        public MainWindowViewModel()
        {
            this.Logout();
        }
        
		/// <summary>
		/// Gets or sets the <see cref="ViewModelBase"/> which should be displayed.
		/// </summary>
        public ViewModelBase MainViewModel
        {
            get 
            { 
                return this.mainViewModel; 
            }

            set
            {
                this.mainViewModel = value;
				OnPropertyChanged("MainViewModel");
            }
        }

        /// <summary>
        /// Gets the command which is executed when a logout should be performed.
        /// </summary>
        public ICommand LogoutCommand
        {
            get
            {
                if (this.logoutCommand == null)
                {
                    this.logoutCommand = new RelayCommand(param => this.Logout());
                }

                return this.logoutCommand;
            }
        }

        /// <summary>
        /// Gets the command which is executed when language should be switched to German.
        /// </summary>
        public ICommand GermanCommand
        {
            get
            {
                if (this.germanCommand == null)
                {
                    this.germanCommand = new RelayCommand(param => App.SwitchCulture("de"));
                }

                return this.germanCommand;
            }
        }

        /// <summary>
        /// Gets the command which is executed when language should be switched to English.
        /// </summary>
        public ICommand EnglishCommand
        {
            get
            {
                if (this.englishCommand == null)
                {
                    this.englishCommand = new RelayCommand(param => App.SwitchCulture("en"));
                }

                return this.englishCommand;
            }
        }

        /// <summary>
        /// Logs out the current user.
        /// </summary>
        private void Logout()
        {
            CurrentUser = null;
            var model = new LoginViewModel();
            model.LoginSuccessful += new System.EventHandler<System.EventArgs>(this.Model_LoginSuccessful);
            this.MainViewModel = model;
        }

        /// <summary>
        /// Handles the LoginSuccessful event of the Model control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Model_LoginSuccessful(object sender, System.EventArgs e)
        {
            (this.MainViewModel as LoginViewModel).LoginSuccessful -= this.Model_LoginSuccessful;
            this.MainViewModel = new GuestbookViewModel();
        }
    }
}